# Copyright 2015 Mirantis inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_utils import uuidutils

from manilaclient import api_versions
from manilaclient import base


class ShareInstance(base.Resource):
    """A share is an extra block level storage to the OpenStack instances."""
    def __repr__(self):
        return "<Share: %s>" % self.id

    def force_delete(self):
        """Delete the specified share ignoring its current state."""
        self.manager.force_delete(self)

    def reset_state(self, state):
        """Update the share with the provided state."""
        self.manager.reset_state(self, state)


class ShareInstanceManager(base.ManagerWithFind):
    """Manage :class:`ShareInstances` resources."""
    resource_class = ShareInstance

    @api_versions.wraps("2.3")
    def get(self, instance):
        """Get a share instance.

        :param instance: either share object or text with its ID.
        :rtype: :class:`ShareInstance`
        """
        share_id = base.getid(instance)
        return self._get("/share_instances/%s" % share_id, "share_instance")

    @api_versions.wraps("2.3", "2.34")
    def list(self, search_opts=None):
        """List all share instances."""
        return self.do_list()

    @api_versions.wraps("2.35")   # noqa
    def list(self, export_location=None, search_opts=None):  # noqa
        """List all share instances."""
        return self.do_list(export_location)

    def do_list(self, export_location=None):
        """List all share instances."""
        path = '/share_instances'
        if export_location:
            if uuidutils.is_uuid_like(export_location):
                path += '?export_location_id=' + export_location
            else:
                path += '?export_location_path=' + export_location

        return self._list(path, 'share_instances')

    def _action(self, action, instance, info=None, **kwargs):
        """Perform a share instance 'action'.

        :param action: text with action name.
        :param instance: either share object or text with its ID.
        :param info: dict with data for specified 'action'.
        :param kwargs: dict with data to be provided for action hooks.
        """
        body = {action: info}
        self.run_hooks('modify_body_for_action', body, **kwargs)
        url = '/share_instances/%s/action' % base.getid(instance)
        return self.api.client.post(url, body=body)

    def _do_force_delete(self, instance, action_name="force_delete"):
        """Delete a share instance forcibly - share status will be avoided.

        :param instance: either share instance object or text with its ID.
        """
        return self._action(action_name, base.getid(instance))

    @api_versions.wraps("2.3", "2.6")
    def force_delete(self, instance):
        return self._do_force_delete(instance, "os-force_delete")

    @api_versions.wraps("2.7")  # noqa
    def force_delete(self, instance):  # noqa
        return self._do_force_delete(instance, "force_delete")

    def _do_reset_state(self, instance, state, action_name):
        """Update the provided share instance with the provided state.

        :param instance: either share object or text with its ID.
        :param state: text with new state to set for share.
        """
        return self._action(action_name, instance, {"status": state})

    @api_versions.wraps("2.3", "2.6")
    def reset_state(self, instance, state):
        return self._do_reset_state(instance, state, "os-reset_status")

    @api_versions.wraps("2.7")  # noqa
    def reset_state(self, instance, state):  # noqa
        return self._do_reset_state(instance, state, "reset_status")



# Copyright 2019 NetApp
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

import ddt

from manilaclient.tests.unit import utils
from manilaclient.tests.unit.v2 import fakes
from manilaclient.v2 import share_network_subnets


@ddt.ddt
class ShareNetworkSubnetTest(utils.TestCase):

    def setUp(self):
        super(ShareNetworkSubnetTest, self).setUp()
        self.manager = share_network_subnets.ShareNetworkSubnetManager(
            fakes.FakeClient())

    def test_create(self):
        share_network_id = 'fake_share_net_id'
        expected_url = share_network_subnets.RESOURCES_PATH % {
            'share_network_id': share_network_id
        }
        expected_values = {
            'neutron_net_id': 'fake_net_id',
            'neutron_subnet_id': 'fake_subnet_id',
            'availability_zone': 'fake_availability_zone',
        }
        expected_body = {'share-network-subnet': expected_values}
        payload = expected_values.copy()
        payload.update({'share_network_id': share_network_id})

        with mock.patch.object(self.manager, '_create', fakes.fake_create):
            result = self.manager.create(**payload)

            self.assertEqual(expected_url, result['url'])
            self.assertEqual(
                share_network_subnets.RESOURCE_NAME,
                result['resp_key'])
            self.assertEqual(
                expected_body,
                result['body'])

    def test_get(self):
        share_network = 'fake_share_network'
        share_subnet = 'fake_share_subnet'

        with mock.patch.object(self.manager, '_get', mock.Mock()):
            self.manager.get(share_network, share_subnet)

            self.manager._get.assert_called_once_with(
                share_network_subnets.RESOURCE_PATH % {
                    'share_network_id': share_network,
                    'share_network_subnet_id': share_subnet
                },
                share_network_subnets.RESOURCE_NAME)

    def test_delete(self):
        share_network = 'fake_share_network'
        share_subnet = 'fake_share_subnet'

        with mock.patch.object(self.manager, '_delete', mock.Mock()):
            self.manager.delete(share_network, share_subnet)

            self.manager._delete.assert_called_once_with(
                share_network_subnets.RESOURCE_PATH % {
                    'share_network_id': share_network,
                    'share_network_subnet_id': share_subnet
                })



#   All rights reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#


from unittest import mock

from osc_lib import exceptions
from osc_lib import utils as oscutils

from manilaclient import api_versions
from manilaclient.osc.v2 import share_servers as osc_share_servers
from manilaclient.tests.unit.osc import osc_utils
from manilaclient.tests.unit.osc.v2 import fakes as manila_fakes


class TestShareServer(manila_fakes.TestShare):

    def setUp(self):
        super(TestShareServer, self).setUp()

        self.servers_mock = self.app.client_manager.share.share_servers
        self.servers_mock.reset_mock()

        self.share_networks_mock = self.app.client_manager.share.share_networks
        self.share_networks_mock.reset_mock()

        self.app.client_manager.share.api_version = api_versions.APIVersion(
            api_versions.MAX_VERSION)


class TestDeleteShareServer(TestShareServer):

    def setUp(self):
        super(TestDeleteShareServer, self).setUp()

        self.share_server = (
            manila_fakes.FakeShareServer.create_one_server())
        self.servers_mock.get.return_value = self.share_server

        self.cmd = osc_share_servers.DeleteShareServer(self.app, None)

    def test_share_server_delete_missing_args(self):
        arglist = []
        verifylist = []

        self.assertRaises(osc_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_share_server_delete(self):
        arglist = [
            self.share_server.id
        ]
        verifylist = [
            ('share_servers', [self.share_server.id])
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.servers_mock.delete.assert_called_once_with(
            self.share_server)
        self.assertIsNone(result)

    def test_share_server_delete_wait(self):
        arglist = [
            self.share_server.id,
            '--wait'
        ]
        verifylist = [
            ('share_servers', [self.share_server.id]),
            ('wait', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        with mock.patch('osc_lib.utils.wait_for_delete', return_value=True):
            result = self.cmd.take_action(parsed_args)

            self.servers_mock.delete.assert_called_once_with(
                self.share_server)
            self.assertIsNone(result)

    def test_share_server_delete_wait_exception(self):
        arglist = [
            self.share_server.id,
            '--wait'
        ]
        verifylist = [
            ('share_servers', [self.share_server.id]),
            ('wait', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        with mock.patch('osc_lib.utils.wait_for_delete', return_value=False):
            self.assertRaises(
                exceptions.CommandError,
                self.cmd.take_action,
                parsed_args
            )


class TestShowShareServer(TestShareServer):

    def setUp(self):
        super(TestShowShareServer, self).setUp()

        self.share_server = (
            manila_fakes.FakeShareServer.create_one_server())
        self.servers_mock.get.return_value = self.share_server

        self.cmd = osc_share_servers.ShowShareServer(self.app, None)

        self.data = tuple(self.share_server._info.values())
        self.columns = tuple(self.share_server._info.keys())

    def test_share_server_show_missing_args(self):
        arglist = []
        verifylist = []

        self.assertRaises(
            osc_utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)

    def test_share_server_show(self):
        arglist = [
            self.share_server.id
        ]
        verifylist = [
            ('share_server', self.share_server.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.servers_mock.get.assert_called_with(
            self.share_server.id
        )

        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)


class TestListShareServer(TestShareServer):

    columns = [
        'ID',
        'Host',
        'Status',
        'Share Network ID',
        'Project ID',
    ]

    def setUp(self):
        super(TestListShareServer, self).setUp()

        self.servers_list = (
            manila_fakes.FakeShareServer.create_share_servers(
                count=2))
        self.servers_mock.list.return_value = self.servers_list

        self.values = (oscutils.get_dict_properties(
            i._info, self.columns) for i in self.servers_list)

        self.cmd = osc_share_servers.ListShareServer(self.app, None)

    def test_list_share_server(self):
        arglist = []
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.servers_mock.list.assert_called_with(search_opts={
            'status': None,
            'host': None,
            'project_id': None,
        })

        self.assertEqual(self.columns, columns)
        self.assertEqual(list(self.values), list(data))

    def test_share_server_list_by_status(self):
        arglist = [
            '--status', self.servers_list[0].status,
        ]
        verifylist = [
            ('status', self.servers_list[0].status),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        search_opts = {
            'status': None,
            'host': None,
            'project_id': None,
        }

        self.servers_mock.list.assert_called_once_with(
            search_opts=search_opts,
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(list(self.values), list(data))


class TestAdoptShareServer(TestShareServer):

    def setUp(self):
        super(TestAdoptShareServer, self).setUp()

        self.share_server = (
            manila_fakes.FakeShareServer.create_one_server(
                attrs={'status': 'available'}
            ))
        self.servers_mock.get.return_value = self.share_server
        self.servers_mock.manage.return_value = self.share_server
        self.share_network_subnets_mock = (
            self.app.client_manager.share.share_network_subnets)

        self.share_network = (
            manila_fakes.FakeShareNetwork.create_one_share_network(
                attrs={'status': 'available'}
            ))
        self.share_network_subnet = (
            manila_fakes.FakeShareNetworkSubnet.create_one_share_subnet())

        self.share_networks_mock.get.return_value = self.share_network
        self.share_network_subnets_mock.get.return_value = (
            self.share_network_subnet)

        self.cmd = osc_share_servers.AdoptShareServer(self.app, None)

        self.data = tuple(self.share_server._info.values())
        self.columns = tuple(self.share_server._info.keys())

    def test_share_server_adopt_missing_args(self):
        arglist = []
        verifylist = []

        self.assertRaises(osc_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_share_server_adopt(self):
        arglist = [
            'somehost@backend',
            self.share_network['id'],
            'share_server_identifier',
            '--share-network-subnet', self.share_network_subnet['id'],
        ]
        verifylist = [
            ('host', 'somehost@backend'),
            ('share_network', self.share_network['id']),
            ('identifier', 'share_server_identifier'),
            ('share_network_subnet', self.share_network_subnet['id']),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)
        self.servers_mock.manage.assert_called_with(
            host='somehost@backend',
            share_network_id=self.share_network['id'],
            identifier='share_server_identifier',
            driver_options={},
            share_network_subnet_id=self.share_network_subnet['id'],
        )
        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)

    def test_share_server_adopt_wait(self):
        arglist = [
            'somehost@backend',
            self.share_network['id'],
            'share_server_identifier',
            '--share-network-subnet', self.share_network_subnet['id'],
            '--wait'
        ]
        verifylist = [
            ('host', 'somehost@backend'),
            ('share_network', self.share_network['id']),
            ('identifier', 'share_server_identifier'),
            ('share_network_subnet', self.share_network_subnet['id']),
            ('wait', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        with mock.patch('osc_lib.utils.wait_for_status', return_value=True):
            self.cmd.take_action(parsed_args)
            self.servers_mock.manage.assert_called_with(
                host='somehost@backend',
                share_network_id=self.share_network['id'],
                identifier='share_server_identifier',
                driver_options={},
                share_network_subnet_id=self.share_network_subnet['id']
            )

    def test_share_server_adopt_subnet_not_supported(self):
        arglist = [
            'somehost@backend',
            self.share_network['id'],
            'share_server_identifier',
            '--share-network-subnet', self.share_network_subnet['id'],
            '--wait'
        ]
        verifylist = [
            ('host', 'somehost@backend'),
            ('share_network', self.share_network['id']),
            ('identifier', 'share_server_identifier'),
            ('share_network_subnet', self.share_network_subnet['id']),
            ('wait', True)
        ]
        self.app.client_manager.share.api_version = api_versions.APIVersion(
            "2.50")

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(exceptions.CommandError,
                          self.cmd.take_action,
                          parsed_args)


class TestAbandonShareServer(TestShareServer):

    def setUp(self):
        super(TestAbandonShareServer, self).setUp()

        self.share_server = (
            manila_fakes.FakeShareServer.create_one_server())
        self.servers_mock.get.return_value = self.share_server

        self.cmd = osc_share_servers.AbandonShareServer(self.app, None)

        self.data = tuple(self.share_server._info.values())
        self.columns = tuple(self.share_server._info.keys())

    def test_share_server_abandon_missing_args(self):
        arglist = []
        verifylist = []

        self.assertRaises(osc_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_share_server_abandon(self):
        arglist = [
            self.share_server.id
        ]
        verifylist = [
            ('share_server', [self.share_server.id])
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.servers_mock.unmanage.assert_called_with(
            self.share_server)
        self.assertIsNone(result)

    def test_share_server_abandon_multiple(self):
        share_servers = (
            manila_fakes.FakeShareServer.create_share_servers(
                count=2))
        arglist = [
            share_servers[0].id,
            share_servers[1].id
        ]
        verifylist = [
            ('share_server', [share_servers[0].id, share_servers[1].id])
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.assertEqual(self.servers_mock.unmanage.call_count,
                         len(share_servers))
        self.assertIsNone(result)

    def test_share_server_abandon_force(self):
        arglist = [
            self.share_server.id,
            '--force'
        ]
        verifylist = [
            ('share_server', [self.share_server.id]),
            ('force', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.servers_mock.unmanage.assert_called_with(
            self.share_server,
            force=True)
        self.assertIsNone(result)

    def test_share_server_abandon_force_exception(self):
        arglist = [
            self.share_server.id,
        ]
        verifylist = [
            ('share_server', [self.share_server.id]),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.servers_mock.unmanage.side_effect = exceptions.CommandError()
        self.assertRaises(exceptions.CommandError,
                          self.cmd.take_action,
                          parsed_args)

    def test_share_server_abandon_wait(self):
        arglist = [
            self.share_server.id,
            '--wait'
        ]
        verifylist = [
            ('share_server', [self.share_server.id]),
            ('wait', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        with mock.patch('osc_lib.utils.wait_for_delete', return_value=True):
            result = self.cmd.take_action(parsed_args)
            self.servers_mock.unmanage.assert_called_with(
                self.share_server)
            self.assertIsNone(result)

    def test_share_server_abandon_wait_error(self):
        arglist = [
            self.share_server.id,
            '--wait'
        ]
        verifylist = [
            ('share_server', [self.share_server.id]),
            ('wait', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        with mock.patch('osc_lib.utils.wait_for_delete', return_value=False):
            self.assertRaises(
                exceptions.CommandError,
                self.cmd.take_action,
                parsed_args)


class TestSetShareServer(TestShareServer):

    def setUp(self):
        super(TestSetShareServer, self).setUp()

        self.share_server = (
            manila_fakes.FakeShareServer.create_one_server(
                methods={'reset_task_state': None}
            )
        )
        self.servers_mock.get.return_value = self.share_server

        self.cmd = osc_share_servers.SetShareServer(self.app, None)

    def test_share_server_set_status(self):
        arglist = [
            self.share_server.id,
            '--status', 'active'
        ]
        verifylist = [
            ('share_server', self.share_server.id),
            ('status', 'active')
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.servers_mock.reset_state.assert_called_with(
            self.share_server,
            parsed_args.status)
        self.assertIsNone(result)

    def test_share_server_set_task_state(self):
        arglist = [
            self.share_server.id,
            '--task-state', 'migration_in_progress'
        ]
        verifylist = [
            ('share_server', self.share_server.id),
            ('task_state', 'migration_in_progress')
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.servers_mock.reset_task_state.assert_called_with(
            self.share_server,
            parsed_args.task_state)
        self.assertIsNone(result)

    def test_share_server_set_status_exception(self):
        arglist = [
            self.share_server.id,
            '--status', 'active'
        ]
        verifylist = [
            ('share_server', self.share_server.id),
            ('status', 'active')
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.servers_mock.reset_state.side_effect = Exception()

        self.assertRaises(
            exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)


class TestShareServerMigrationCancel(TestShareServer):

    def setUp(self):
        super(TestShareServerMigrationCancel, self).setUp()

        self.share_server = (
            manila_fakes.FakeShareServer.create_one_server(
                attrs={
                    'status': 'migrating',
                },
                methods={'migration_cancel': None}
            )
        )
        self.servers_mock.get.return_value = self.share_server

        # Get the command objects to test
        self.cmd = osc_share_servers.ShareServerMigrationCancel(self.app, None)

    def test_share_server_migration_cancel(self):
        arglist = [
            self.share_server.id
        ]
        verifylist = [
            ('share_server', self.share_server.id)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.share_server.migration_cancel.assert_called


class TestShareServerMigrationComplete(TestShareServer):

    def setUp(self):
        super(TestShareServerMigrationComplete, self).setUp()

        self.share_server = (
            manila_fakes.FakeShareServer.create_one_server(
                attrs={
                    'status': 'migrating',
                },
                methods={'migration_complete': None}
            )
        )
        self.servers_mock.get.return_value = self.share_server

        # Get the command objects to test
        self.cmd = osc_share_servers.ShareServerMigrationComplete(
            self.app, None)

    def test_share_server_migration_complete(self):
        arglist = [
            self.share_server.id
        ]
        verifylist = [
            ('share_server', self.share_server.id)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.share_server.migration_complete.assert_called


class TestShareServerMigrationShow(TestShareServer):

    def setUp(self):
        super(TestShareServerMigrationShow, self).setUp()

        self.new_share_network = manila_fakes.FakeShareNetwork \
            .create_one_share_network()
        self.share_networks_mock.get.return_value = self.new_share_network

        self.share_server = (
            manila_fakes.FakeShareServer.create_one_server(
                attrs={
                    'status': 'migrating',
                    'task_state': 'migration_in_progress'
                },
                methods={'migration_get_progress': None}
            )
        )
        self.servers_mock.get.return_value = self.share_server

        # Get the command objects to test
        self.cmd = osc_share_servers.ShareServerMigrationShow(self.app, None)

    def test_share_server_migration_show(self):
        arglist = [
            self.share_server.id
        ]
        verifylist = [
            ('share_server', self.share_server.id)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.share_server.migration_get_progress.assert_called


class TestShareServerMigrationStart(TestShareServer):
    def setUp(self):
        super(TestShareServerMigrationStart, self).setUp()

        self.new_share_network = manila_fakes.FakeShareNetwork \
            .create_one_share_network()
        self.share_networks_mock.get.return_value = self.new_share_network

        self.share_server = (
            manila_fakes.FakeShareServer.create_one_server(
                attrs={
                    'check_only': 'False',
                },
                methods={'migration_start': None, 'migration_check': None}
            ))
        self.servers_mock.get.return_value = self.share_server

        # Get the command objects to test
        self.cmd = osc_share_servers.ShareServerMigrationStart(self.app, None)

    def test_share_server_migration_start_with_new_share_network(self):
        """Test share server migration with new_share_network"""

        arglist = [
            '1234',
            'host@backend',
            '--preserve-snapshots', 'False',
            '--writable', 'False',
            '--nondisruptive', 'False',
            '--new-share-network', self.new_share_network.id
        ]

        verifylist = [
            ('share_server', '1234'),
            ('host', 'host@backend'),
            ('preserve_snapshots', 'False'),
            ('writable', 'False'),
            ('nondisruptive', 'False'),
            ('new_share_network', self.new_share_network.id)
        ]

        self.app.client_manager.share.api_version = api_versions.APIVersion(
            "2.57")
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.share_server.migration_start.assert_called_with(
            'host@backend',
            'False',
            'False',
            'False',
            self.new_share_network.id
        )
        self.assertEqual(result, ({}, {}))

    def test_share_server_migration_start_with_check_only(self):
        """Test share server migration start with check only"""

        arglist = [
            '1234',
            'host@backend',
            '--preserve-snapshots', 'True',
            '--writable', 'True',
            '--nondisruptive', 'False',
            '--new-share-network', self.new_share_network.id,
            '--check-only',
        ]

        verifylist = [
            ('share_server', '1234'),
            ('host', 'host@backend'),
            ('preserve_snapshots', 'True'),
            ('writable', 'True'),
            ('nondisruptive', 'False'),
            ('new_share_network', self.new_share_network.id),
            ('check_only', True)
        ]

        expected_result = {
            'compatible': True,
            'requested_capabilities': {
                'writable': 'True',
                'nondisruptive': 'False',
                'preserve_snapshots': 'True',
                'share_network_id': None,
                'host': 'host@backend'
            },
            'supported_capabilities': {
                'writable': True,
                'nondisruptive': False,
                'preserve_snapshots': True,
                'share_network_id': self.new_share_network.id,
                'migration_cancel': True,
                'migration_get_progress': True
            }
        }

        self.share_server.migration_check.return_value = expected_result

        self.app.client_manager.share.api_version = api_versions.APIVersion(
            "2.57")
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.share_server.migration_check.assert_called_with(
            'host@backend',
            'True',
            'False',
            'True',
            self.new_share_network.id,

        )
        result_dict = {}
        for count, column in enumerate(columns):
            result_dict[column] = data[count]
        self.assertEqual(expected_result, result_dict)

    def test_share_server_migration_start_with_api_version_exception(self):
        """Test share server migration start with API microversion exception"""

        self.app.client_manager.share.api_version = api_versions.APIVersion(
            "2.50")
        arglist = [
            '1234',
            'host@backend',
            '--preserve-snapshots', 'False',
            '--writable', 'False',
            '--nondisruptive', 'False',
            '--new-share-network', self.new_share_network.id
        ]

        verifylist = [
            ('share_server', '1234'),
            ('host', 'host@backend'),
            ('preserve_snapshots', 'False'),
            ('writable', 'False'),
            ('nondisruptive', 'False'),
            ('new_share_network', self.new_share_network.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(
            exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import logging

from openstackclient.identity import common as identity_common
from osc_lib.cli import parseractions
from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils as osc_utils

from manilaclient import api_versions
from manilaclient.common._i18n import _

LOG = logging.getLogger(__name__)


class CreateShareGroup(command.ShowOne):
    """Create new share group."""
    _description = _(
        "Create new share group")

    def get_parser(self, prog_name):
        parser = super(CreateShareGroup, self).get_parser(prog_name)
        parser.add_argument(
            '--name',
            metavar="<name>",
            default=None,
            help=_('Share group name')
        )
        parser.add_argument(
            "--description",
            metavar="<description>",
            default=None,
            help=_("Share group description."),
        )
        parser.add_argument(
            "--share-types",
            metavar="<share-types>",
            nargs="+",
            default=[],
            help=_("Name or ID of share type(s)."),
        )
        parser.add_argument(
            "--share-group-type",
            metavar="<share-group-type>",
            default=None,
            help=_("Share group type name or ID of the share "
                   "group to be created."),
        )
        parser.add_argument(
            "--share-network",
            metavar="<share-network>",
            default=False,
            help=_("Specify share network name or id"),
        )
        parser.add_argument(
            "--source-share-group-snapshot",
            metavar="<source-share-group-snapshot>",
            default=False,
            help=_("Share group snapshot name or ID to create "
                   "the share group from."),
        )
        parser.add_argument(
            "--availability-zone",
            metavar='<availability-zone>',
            default=None,
            help=_("Optional availability zone in which group "
                   "should be created"),
        )
        parser.add_argument(
            "--wait",
            action='store_true',
            default=False,
            help=_('Wait for share group creation')
        )
        return parser

    def take_action(self, parsed_args):
        share_client = self.app.client_manager.share

        share_types = []
        for share_type in parsed_args.share_types:
            share_types.append(
                osc_utils.find_resource(
                    share_client.share_types,
                    share_type,
                )
            )
        share_group_type = None
        if parsed_args.share_group_type:
            share_group_type = osc_utils.find_resource(
                share_client.share_group_types,
                parsed_args.share_group_type).id

        share_network = None
        if parsed_args.share_network:
            share_network = osc_utils.find_resource(
                share_client.share_networks,
                parsed_args.share_network).id

        source_share_group_snapshot = None
        if parsed_args.source_share_group_snapshot:
            source_share_group_snapshot = osc_utils.find_resource(
                share_client.source_share_group_snapshots,
                parsed_args.source_share_group_snapshot).id

        body = {
            'name': parsed_args.name,
            'description': parsed_args.description,
            'share_types': share_types,
            'share_group_type': share_group_type,
            'share_network': share_network,
            'source_share_group_snapshot': source_share_group_snapshot,
            'availability_zone': parsed_args.availability_zone
        }

        share_group = share_client.share_groups.create(**body)

        if parsed_args.wait:
            if not osc_utils.wait_for_status(
                status_f=share_client.share_groups.get,
                res_id=share_group.id,
                success_status=['available']
            ):
                LOG.error(_("ERROR: Share group is in error state."))

            share_group = osc_utils.find_resource(
                share_client.share_groups,
                share_group.id)

        printable_share_group = share_group._info
        printable_share_group.pop('links', None)

        if printable_share_group.get('share_types'):
            if parsed_args.formatter == 'table':
                printable_share_group['share_types'] = (
                    "\n".join(printable_share_group['share_types'])
                )

        return self.dict2columns(printable_share_group)


class DeleteShareGroup(command.Command):
    """Delete one or more share groups."""
    _description = _("Delete one or more share groups")

    def get_parser(self, prog_name):
        parser = super(DeleteShareGroup, self).get_parser(prog_name)
        parser.add_argument(
            "share_group",
            metavar="<share_group>",
            nargs="+",
            help=_("Name or ID of the share group(s) to delete")
        )
        parser.add_argument(
            "--force",
            action='store_true',
            default=False,
            help=_("Attempt to force delete the share group (Default=False) "
                   "(Admin only).")
        )
        parser.add_argument(
            "--wait",
            action='store_true',
            default=False,
            help=_("Wait for share group to delete")
        )
        return parser

    def take_action(self, parsed_args):
        share_client = self.app.client_manager.share
        result = 0

        for share_group in parsed_args.share_group:
            try:
                share_group_obj = osc_utils.find_resource(
                    share_client.share_groups,
                    share_group)

                share_client.share_groups.delete(
                    share_group_obj,
                    force=parsed_args.force)

                if parsed_args.wait:
                    if not osc_utils.wait_for_delete(
                            manager=share_client.share_groups,
                            res_id=share_group_obj.id):
                        result += 1

            except Exception as e:
                result += 1
                LOG.error(_(
                    "Failed to delete share group with "
                    "name or ID '%(share_group)s': %(e)s"),
                    {'share_group': share_group, 'e': e})

        if result > 0:
            total = len(parsed_args.share_group)
            msg = (_("%(result)s of %(total)s share groups failed "
                   "to delete.") % {'result': result, 'total': total})
            raise exceptions.CommandError(msg)


class ListShareGroup(command.Lister):
    """List share groups."""
    _description = _("List share groups")

    def get_parser(self, prog_name):
        parser = super(ListShareGroup, self).get_parser(prog_name)
        parser.add_argument(
            "--all-projects",
            action='store_true',
            default=False,
            help=_("Display share groups from all projects (Admin only).")
        )
        parser.add_argument(
            "--name",
            metavar="<name>",
            default=None,
            help=_("Filter results by name.")
        )
        parser.add_argument(
            "--description",
            metavar="<description>",
            default=None,
            help=_("Filter results by description. Available "
                   "only for microversion >= 2.36.")
        )
        parser.add_argument(
            "--status",
            metavar="<status>",
            default=None,
            help=_("Filter results by status.")
        )
        parser.add_argument(
            "--share-server",
            metavar="<share-server-id>",
            default=None,
            help=_("Filter results by share server ID.")
        )
        parser.add_argument(
            "--share-group-type",
            metavar="<share-group-type>",
            default=None,
            help=_("Filter results by a share group type ID "
                   "or name that was used for share group "
                   "creation. ")
        )
        parser.add_argument(
            "--snapshot",
            metavar="<snapshot>",
            default=None,
            help=_("Filter results by share group snapshot "
                   "name or ID that was used to create the "
                   "share group. ")
        )
        parser.add_argument(
            "--host",
            metavar="<host>",
            default=None,
            help=_("Filter results by host.")
        )
        parser.add_argument(
            "--share-network",
            metavar="<share-network>",
            default=None,
            help=_("Filter results by share-network name or "
                   "ID. ")
        )
        parser.add_argument(
            "--project",
            metavar="<project>",
            default=None,
            help=_("Filter results by project name or ID. Useful with "
                   "set key '--all-projects'. ")
        )
        identity_common.add_project_domain_option_to_parser(parser)
        parser.add_argument(
            "--limit",
            metavar="<limit>",
            type=int,
            default=None,
            action=parseractions.NonNegativeAction,
            help=_("Limit the number of share groups returned")
        )
        parser.add_argument(
            "--marker",
            metavar="<marker>",
            help=_("The last share group ID of the previous page")
        )
        parser.add_argument(
            '--sort',
            metavar="<key>[:<direction>]",
            default='name:asc',
            help=_("Sort output by selected keys and directions(asc or desc) "
                   "(default: name:asc), multiple keys and directions can be "
                   "specified separated by comma")
        )
        parser.add_argument(
            "--name~",
            metavar="<name~>",
            default=None,
            help=_("Filter results matching a share group "
                   "name pattern. Available only for "
                   "microversion >= 2.36. ")
        )
        parser.add_argument(
            "--description~",
            metavar="<description~>",
            default=None,
            help=_("Filter results matching a share group "
                   "description pattern. Available only for "
                   "microversion >= 2.36. ")
        )
        return parser

    def take_action(self, parsed_args):
        share_client = self.app.client_manager.share
        identity_client = self.app.client_manager.identity

        share_server_id = None
        if parsed_args.share_server:
            share_server_id = osc_utils.find_resource(
                share_client.share_servers,
                parsed_args.share_server).id

        share_group_type = None
        if parsed_args.share_group_type:
            share_group_type = osc_utils.find_resource(
                share_client.share_group_types,
                parsed_args.share_group_type).id

        snapshot = None
        if parsed_args.snapshot:
            snapshot = apiutils.find_resource(
                share_client.share_snapshots,
                parsed_args.snapshot).id

        share_network = None
        if parsed_args.share_network:
            share_network = osc_utils.find_resource(
                share_client.share_networks,
                parsed_args.share_network).id

        project_id = None
        if parsed_args.project:
            project_id = identity_common.find_project(
                identity_client,
                parsed_args.project,
                parsed_args.project_domain).id

        columns = [
            'ID',
            'Name',
            'Status',
            'Description',
        ]

        search_opts = {
            'all_tenants': parsed_args.all_projects,
            'name': parsed_args.name,
            'status': parsed_args.status,
            'share_server_id': share_server_id,
            'share_group_type': share_group_type,
            'snapshot': snapshot,
            'host': parsed_args.host,
            'share_network': share_network,
            'project_id': project_id,
            'limit': parsed_args.limit,
            'offset': parsed_args.marker,
        }

        if share_client.api_version >= api_versions.APIVersion("2.36"):
            search_opts['name~'] = getattr(parsed_args, 'name~')
            search_opts['description~'] = getattr(parsed_args, 'description~')
            search_opts['description'] = parsed_args.description
        elif (parsed_args.description or getattr(parsed_args, 'name~') or
              getattr(parsed_args, 'description~')):
            raise exceptions.CommandError(
                "Pattern based filtering (name~, description~ and description)"
                " is only available with manila API version >= 2.36")

        if parsed_args.all_projects:
            columns.append('Project ID')
        share_groups = share_client.share_groups.list(search_opts=search_opts)

        data = (osc_utils.get_dict_properties(
            share_group._info, columns) for share_group in share_groups)

        return (columns, data)


class ShowShareGroup(command.ShowOne):
    """Show share group."""
    _description = _("Show details about a share group")

    def get_parser(self, prog_name):
        parser = super(ShowShareGroup, self).get_parser(prog_name)
        parser.add_argument(
            "share_group",
            metavar="<share-group>",
            help=_("Name or ID of the share group.")
        )
        return parser

    def take_action(self, parsed_args):
        share_client = self.app.client_manager.share

        share_group = osc_utils.find_resource(
            share_client.share_groups,
            parsed_args.share_group)

        printable_share_group = share_group._info
        printable_share_group.pop('links', None)

        if printable_share_group.get('share_types'):
            if parsed_args.formatter == 'table':
                printable_share_group['share_types'] = "\n".join(
                    printable_share_group['share_types'])

        return self.dict2columns(printable_share_group)


class SetShareGroup(command.Command):
    """Set share group."""
    _description = _("Explicitly set share group status")

    def get_parser(self, prog_name):
        parser = super(SetShareGroup, self).get_parser(prog_name)
        parser.add_argument(
            'share_group',
            metavar="<share-group>",
            help=_('Name or ID of the share group to update.')
        )
        parser.add_argument(
            '--name',
            metavar="<name>",
            default=None,
            help=_('New name for the share group. (Default=None)')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            default=None,
            help=_('Share group description. (Default=None)')
        )
        parser.add_argument(
            '--status',
            metavar='<status>',
            default=None,
            help=_('Explicitly update the status of a share group (Admin  '
                   'only). Examples include: available, error, creating, '
                   'deleting, error_deleting.')
        )
        return parser

    def take_action(self, parsed_args):
        share_client = self.app.client_manager.share
        result = 0

        share_group = osc_utils.find_resource(
            share_client.share_groups,
            parsed_args.share_group)

        kwargs = {}
        if parsed_args.name is not None:
            kwargs['name'] = parsed_args.name
        if parsed_args.description is not None:
            kwargs['description'] = parsed_args.description
        if kwargs:
            try:
                share_client.share_groups.update(
                    share_group.id,
                    **kwargs
                )
            except Exception as e:
                LOG.error(_("Failed to update share group name "
                          "or description: %s"), e)
                result += 1
        if parsed_args.status:
            try:
                share_group.reset_state(parsed_args.status)
            except Exception as e:
                LOG.error(_(
                    "Failed to set status for the share group: %s"), e)
                result += 1

        if result > 0:
            raise exceptions.CommandError(_("One or more of the "
                                          "set operations failed"))


class UnsetShareGroup(command.Command):
    """Unset a share group property."""
    _description = _("Unset a share group property")

    def get_parser(self, prog_name):
        parser = super(UnsetShareGroup, self).get_parser(prog_name)
        parser.add_argument(
            'share_group',
            metavar="<share-group>",
            help=_("Name or ID of the share group "
                   "to set a property for.")
        )
        parser.add_argument(
            "--name",
            action='store_true',
            help=_("Unset share group name."),
        )
        parser.add_argument(
            "--description",
            action='store_true',
            help=_("Unset share group description."),
        )
        return parser

    def take_action(self, parsed_args):
        share_client = self.app.client_manager.share

        share_group = osc_utils.find_resource(
            share_client.share_groups,
            parsed_args.share_group)

        kwargs = {}
        if parsed_args.name:
            kwargs['name'] = None
        if parsed_args.description:
            kwargs['description'] = None
        if kwargs:
            try:
                share_client.share_groups.update(
                    share_group,
                    **kwargs
                )
            except Exception as e:
                raise exceptions.CommandError(_(
                    "Failed to unset share_group name "
                    "or description : %s" % e))



# Copyright 2014 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

import ddt

from manilaclient import base
from manilaclient.common import constants
from manilaclient.tests.unit import utils
from manilaclient.tests.unit.v2 import fakes
from manilaclient.v2 import share_servers


class FakeShareServer(object):
    _info = {
        "backend_details": {
            "fake_key1": "fake_value1",
            "fake_key2": "fake_value2",
        }
    }


class ShareServerTest(utils.TestCase):

    def setUp(self):
        super(ShareServerTest, self).setUp()
        self.share_server_id = 'foo'
        self.share_network = 'bar'
        info = {
            'id': self.share_server_id,
            'share_network_name': self.share_network,
        }
        self.resource_class = share_servers.ShareServer(
            manager=self, info=info)

    def test_get_repr_of_share_server(self):
        self.assertIn(
            'ShareServer: %s' % self.share_server_id,
            repr(self.resource_class),
        )

    def test_get_share_network_attr(self):
        # We did not set 'share_network' attr within instance, it is expected
        # that attr 'share_network_name' will be reused.
        self.assertEqual(self.resource_class.share_network, self.share_network)

    def test_get_nonexistent_share_network_name(self):
        resource_class = share_servers.ShareServer(manager=self, info={})
        try:
            # We expect AttributeError instead of endless loop of getattr
            resource_class.share_network_name
        except AttributeError:
            pass
        else:
            raise Exception("Expected exception 'AttributeError' "
                            "has not been raised.")


@ddt.ddt
class ShareServerManagerTest(utils.TestCase):

    def setUp(self):
        super(ShareServerManagerTest, self).setUp()
        self.manager = share_servers.ShareServerManager(api=fakes.FakeClient())

    def test_list(self):
        with mock.patch.object(self.manager, '_list',
                               mock.Mock(return_value=None)):
            self.manager.list()
            self.manager._list.assert_called_once_with(
                share_servers.RESOURCES_PATH,
                share_servers.RESOURCES_NAME)

    @ddt.data(None, {}, {'opt1': 'fake_opt1', 'opt12': 'fake_opt2'})
    def test_manage(self, driver_options):
        host = 'fake_host'
        share_network_id = 'fake_share_net_id'
        share_network_subnet_id = 'fake_share_network_subnet_id'
        identifier = 'ff-aa-kk-ee-00'
        if driver_options is None:
            driver_options = {}
        expected_body = {
            'host': host,
            'share_network_id': share_network_id,
            'identifier': identifier,
            'driver_options': driver_options,
            'share_network_subnet_id': share_network_subnet_id,
        }
        with mock.patch.object(self.manager, '_create',
                               mock.Mock(return_value='fake')):
            result = self.manager.manage(
                host, share_network_id, identifier,
                driver_options=driver_options,
                share_network_subnet_id=share_network_subnet_id)
            self.manager._create.assert_called_once_with(
                share_servers.RESOURCES_PATH + '/manage',
                {'share_server': expected_body}, 'share_server'
            )
            self.assertEqual('fake', result)

    @ddt.data(True, False)
    def test_unmanage(self, force):
        share_server = {'id': 'fake'}
        with mock.patch.object(self.manager, '_action',
                               mock.Mock(return_value='fake')):
            result = self.manager.unmanage(share_server, force)
            self.manager._action.assert_called_once_with(
                "unmanage", share_server, {'force': force})

            self.assertEqual('fake', result)

    def test_reset_state(self):
        share_server = {'id': 'fake'}
        state = constants.STATUS_AVAILABLE
        with mock.patch.object(self.manager, '_action',
                               mock.Mock(return_value='fake')):
            result = self.manager.reset_state(share_server, state)
            self.manager._action.assert_called_once_with(
                "reset_status", share_server, {"status": state})
            self.assertEqual('fake', result)

    @ddt.data(("reset_status", {"status": constants.STATUS_AVAILABLE}),
              ("unmanage", {"id": "fake_id"}))
    @ddt.unpack
    def test__action(self, action, info):
        action = ""
        share_server = {"id": 'fake_id'}
        expected_url = '/share-servers/%s/action' % share_server['id']
        expected_body = {action: info}

        with mock.patch.object(self.manager.api.client, 'post',
                               mock.Mock(return_value='fake')):
            self.mock_object(base, 'getid',
                             mock.Mock(return_value=share_server['id']))
            result = self.manager._action(action, share_server, info)
            self.manager.api.client.post.assert_called_once_with(
                expected_url, body=expected_body
            )
            self.assertEqual('fake', result)

    def test_list_with_one_search_opt(self):
        host = 'fake_host'
        query_string = "?host=%s" % host
        with mock.patch.object(self.manager, '_list',
                               mock.Mock(return_value=None)):
            self.manager.list({'host': host})
            self.manager._list.assert_called_once_with(
                share_servers.RESOURCES_PATH + query_string,
                share_servers.RESOURCES_NAME,
            )

    def test_list_with_two_search_opts(self):
        host = 'fake_host'
        status = 'fake_status'
        query_string = "?host=%s&status=%s" % (host, status)
        with mock.patch.object(self.manager, '_list',
                               mock.Mock(return_value=None)):
            self.manager.list({'host': host, 'status': status})
            self.manager._list.assert_called_once_with(
                share_servers.RESOURCES_PATH + query_string,
                share_servers.RESOURCES_NAME,
            )

    def test_delete(self):
        share_server_id = 'fake_share_server_id'
        with mock.patch.object(self.manager, '_delete', mock.Mock()):
            self.manager.delete(share_server_id)
            self.manager._delete.assert_called_once_with(
                share_servers.RESOURCE_PATH % share_server_id)

    def test_get(self):
        server = FakeShareServer()
        with mock.patch.object(self.manager, '_get',
                               mock.Mock(return_value=server)):
            share_server_id = 'fake_share_server_id'
            self.manager.get(share_server_id)
            self.manager._get.assert_called_once_with(
                "%s/%s" % (share_servers.RESOURCES_PATH, share_server_id),
                share_servers.RESOURCE_NAME)
            for key in ["details:fake_key1", "details:fake_key2"]:
                self.assertIn(key, list(server._info))

    def test_details(self):
        with mock.patch.object(self.manager, '_get',
                               mock.Mock(return_value=None)):
            share_server_id = 'fake_share_server_id'
            self.manager.details(share_server_id)
            self.manager._get.assert_called_once_with(
                "%s/%s/details" % (share_servers.RESOURCES_PATH,
                                   share_server_id), 'details')

    def test_migration_check(self):
        share_server = "fake_share_server"
        host = "fake_host"
        returned = {
            'compatible': True,
            'capacity': True,
            'capability': True,
            'writable': True,
            'nondisruptive': True,
            'preserve_snapshots': True,
        }

        with mock.patch.object(self.manager, "_action",
                               mock.Mock(return_value=['200', returned])):
            result = self.manager.migration_check(
                share_server, host, writable=True, nondisruptive=True,
                preserve_snapshots=True)
            self.manager._action.assert_called_once_with(
                'migration_check', share_server, {
                    "host": host,
                    "writable": True,
                    "nondisruptive": True,
                    "preserve_snapshots": True,
                    "new_share_network_id": None,
                })

            self.assertEqual(returned, result)

    def test_migration_start(self):
        share_server = "fake_share_server"
        host = "fake_host"
        returned = "fake"

        with mock.patch.object(self.manager, "_action",
                               mock.Mock(return_value=returned)):
            result = self.manager.migration_start(
                share_server, host, writable=True, nondisruptive=True,
                preserve_snapshots=True)
            self.manager._action.assert_called_once_with(
                'migration_start', share_server, {
                    "host": host,
                    "writable": True,
                    "nondisruptive": True,
                    "preserve_snapshots": True,
                    "new_share_network_id": None,
                })

            self.assertEqual(returned, result)

    def test_migration_complete(self):
        share_server = "fake_share_server"
        returned = "fake"

        with mock.patch.object(self.manager, "_action",
                               mock.Mock(return_value=['200', returned])):
            result = self.manager.migration_complete(share_server)

            self.manager._action.assert_called_once_with(
                "migration_complete", share_server)
            self.assertEqual(returned, result)

    def test_migration_get_progress(self):
        share_server = "fake_share_server"
        returned = "fake"

        with mock.patch.object(self.manager, "_action",
                               mock.Mock(return_value=['200', returned])):
            result = self.manager.migration_get_progress(share_server)

            self.manager._action.assert_called_once_with(
                "migration_get_progress", share_server)
            self.assertEqual(returned, result)

    def test_reset_task_state(self):
        share_server = "fake_share_server"
        state = "fake_state"
        returned = "fake"

        with mock.patch.object(self.manager, "_action",
                               mock.Mock(return_value=returned)):
            result = self.manager.reset_task_state(share_server, state)

            self.manager._action.assert_called_once_with(
                "reset_task_state", share_server, {'task_state': state})
            self.assertEqual(returned, result)

    def test_migration_cancel(self):
        share_server = "fake_share_server"
        returned = "fake"

        with mock.patch.object(self.manager, "_action",
                               mock.Mock(return_value=returned)):
            result = self.manager.migration_cancel(share_server)

            self.manager._action.assert_called_once_with(
                "migration_cancel", share_server)
            self.assertEqual(returned, result)



# Copyright 2010 Jacob Kaplan-Moss
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
Exception definitions.
"""

from manilaclient.common._i18n import _
from manilaclient.common.apiclient.exceptions import *  # noqa


class ManilaclientException(Exception):
    """A generic client error."""
    message = _("An unexpected error occured.")

    def __init__(self, message):
        self.message = message or self.message

    def __str__(self):
        return self.message


class ResourceInErrorState(ManilaclientException):
    """A resource is in an unexpected 'error' state."""
    message = _("Resource is in error state")


class TimeoutException(ManilaclientException):
    """A request has timed out"""
    message = _("Request has timed out")


class NoTokenLookupException(ClientException):  # noqa: F405
    """No support for looking up endpoints.

    This form of authentication does not support looking up
    endpoints from an existing token.
    """
    pass


class VersionNotFoundForAPIMethod(Exception):
    msg_fmt = "API version '%(vers)s' is not supported on '%(method)s' method."

    def __init__(self, version, method):
        self.version = version
        self.method = method

    def __str__(self):
        return self.msg_fmt % {"vers": self.version, "method": self.method}
